/* listfile.h -- display a long listing of a file
   Copyright (C) 2007, 2008, 2010, 2011 Free Software Foundation, Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#if !defined DIRCALLBACK_H
# define DIRCALLBACK_H

struct saved_cwd;

int run_in_dirfd (int fd, int (*callback)(void*), void *usercontext);
int run_in_dir (const struct saved_cwd*, int (*callback)(void*), void *usercontext);

#endif
